clear all
global cGains DEADBAND
figure(1)
N = 8;
PACKETLOSS = 0.05; %packet loss rate (assume packets are randomly lost
% 
GOAL = [100 300 160000 40000 40000];
% GOAL = [ 1200 0 160000 40000 40000];

% GOAL = [ 0 0 160000 40000 40000];
% GOAL = [ 2000 0 40000 0 160000];
% GOAL = [ 10000 0 40000 40000 160000];
% GOAL = [ 0 0 300000 0 0];

DEADBAND = 0;
COMMR = Inf;

KP = 0.7;
KI = 0.1;
GAMMA = 0.05;
cGains = diag([1.0, 1.0, 0.00001,0.00002,0.00001]);
dt=0.4;
STEPS = 10000;


u_vel_list = zeros(2,STEPS);
x_vel_comp_list = zeros(5,STEPS);
y_vel_comp_list = zeros(5,STEPS);

global WHEELBASE WHEELRADIUS POINT_OFFSET MAX_WHEEL_V_TICKS STEPS_PER_REV
WHEELRADIUS = 20.6; % in mm, approximately
WHEELBASE = 26.7;  % half the distance between the wheels, approximately (in mm)
POINT_OFFSET = 35.0;
MAX_WHEEL_V_TICKS = 750;
STEPS_PER_REV = 2000;  % stepper motor steps per wheel revolution

% MAXWHEELSPEED = 2.0*PI; % in rad/sec, based on max of 1 step every 2 interrupts, 0.5 ms between interrupts, 1000 steps per rev
% MAXLINVEL (MAXWHEELSPEED*WHEELRADIUS)  % max lin vel of robot in mm/s
% MAXANGVEL (MAXLINVEL/WHEELBASE)  % max ang vel about center in rad/s


for i=1:N
    bots(i) = struct('pos', [0; 0; 0],...
        'u_vel', [0; 0],...
        'wheel_vel', [0; 0],...
        'u_i', [0; 0; 0; 0; 0],...
        'x_i', [0; 0; 0; 0; 0],...
        'w_i', [0; 0; 0; 0; 0],...
        'x_sum', [0; 0; 0; 0; 0],...
        'w_sum', [0; 0; 0; 0; 0],...
        'packets_rxed', 0);
    
    bots(i).pos(1:2) = (rand(2,1)-0.5)*2000 + GOAL(1:2)';
end

for step = 1:STEPS
    disp(step)
    
    if step == 300
        GOAL = [ -800 0 30000 0 150000];
    elseif step == 600
        GOAL = [ -600 0 40000 0 40000];
    elseif step == 900
        GOAL = [ 0 0 160000 0 20000];
    elseif step == 1200
        GOAL = [ 600 0 40000 0 40000];
    elseif step == 1500
        GOAL = [ 800 0 30000 0 150000];
    end   
%     
    
    %send data out to other agents
    for i = 1:N
        for j = 1:N
            if i~=j
                if (rand>PACKETLOSS) && ...
                        COMMR>(sqrt((bots(i).pos(1)-bots(j).pos(1))^2 + (bots(i).pos(2)-bots(j).pos(2))^2))
                    bots(j).x_sum = bots(j).x_sum + (bots(j).x_i - bots(i).x_i);
                    bots(j).w_sum = bots(j).w_sum + (bots(j).w_i - bots(i).w_i);
                    bots(j).packets_rxed = bots(j).packets_rxed+1;
                end
            end
        end
    end
    
    %Calculate new consensus
    for i = 1:N
        bots(i).u_i(1)=bots(i).pos(1); 
        bots(i).u_i(2)=bots(i).pos(2); 
        bots(i).u_i(3)=(bots(i).pos(1) - bots(i).x_i(1))^2; 
        bots(i).u_i(4)=(bots(i).pos(1) - bots(i).x_i(1))*(bots(i).pos(2) - bots(i).x_i(2));
        bots(i).u_i(5)=(bots(i).pos(2) - bots(i).x_i(2))^2; 
        if bots(i).packets_rxed == 0
            bots(i).packets_rxed = 1; %avoid div-by-zero; x_sum and w_sum are 0 anyway so no difference.
        end
        bots(i).x_i = bots(i).x_i...
            + GAMMA*(bots(i).u_i-bots(i).x_i)...
            - KP/bots(i).packets_rxed*bots(i).x_sum...
            + KI/bots(i).packets_rxed*bots(i).w_sum;
        bots(i).w_i = bots(i).w_i - KI/bots(i).packets_rxed*bots(i).x_sum;
        bots(i).x_sum=0;
        bots(i).w_sum=0;
        bots(i).packets_rxed = 0;     
    end
    
    %Calculate motion vector
     for i = 1:N
        [bots(i).u_vel vel_comps] = ...
                 calcMotionVectorCentralM(bots(i).pos, bots(i).x_i, GOAL);
        bots(i).wheel_vel = wheelspeed(bots(i).pos, bots(i).u_vel);
        bots(i).pos = updatePos(bots(i).pos, bots(i).wheel_vel,dt/2); 
     end
     
     %in our swarm we calculate motion control every 0.2 seconds and send
     %data every 0.4 seconds.  Here we used dt = 0.4
     for i = 1:N
        [bots(i).u_vel vel_comps] = ...
                 calcMotionVectorCentralM(bots(i).pos, bots(i).x_i, GOAL);
        bots(i).wheel_vel = wheelspeed(bots(i).pos, bots(i).u_vel);
        bots(i).pos = updatePos(bots(i).pos, bots(i).wheel_vel,dt/2); 
     end   
     
%      x_vel_comp_list(1:5,step) = vel_comps(1,:)';
%      y_vel_comp_list(1:5,step) = vel_comps(2,:)';
%     u_vel_list(1:2,step) = bots(1).u_vel;

    %draw GOAL ellipse

    xlim([-1000 1000]);
    ylim([-1000 1000]);
    grid on 
    axis equal
    hold on
    
    drawellipse(GOAL(1), GOAL(2), GOAL(3), GOAL(4), GOAL(5),'r');
    for i = 1:N
        scatter(bots(i).pos(1), bots(i).pos(2));
        line([bots(i).pos(1), bots(i).pos(1) + bots(i).u_vel(1)],...
             [bots(i).pos(2), bots(i).pos(2) + bots(i).u_vel(2)]);
        drawellipse_fast(bots(i).x_i(1),bots(i).x_i(2),...
            bots(i).x_i(3),...
            bots(i).x_i(4),...
            bots(i).x_i(5),'k');
    end
    drawnow;
    delete(gca);
    hold off

end
    

