function [wheel_speed] = wheelspeed(pos,u_vel)
global  WHEELBASE POINT_OFFSET MAX_WHEEL_V_TICKS DEADBAND
u_x = u_vel(1);
u_y = u_vel(2);
theta = pos(3);
CTheta = cos(theta);
STheta = sin(theta);

if (sqrt(u_x^2 + u_y^2)<DEADBAND);
    u_x = 0;
    u_y = 0;
end

uX_R = CTheta*u_x + STheta*u_y;    
uY_R = -STheta*u_x + CTheta*u_y;

% 	// translate velocity vector into wheel speeds
vL_f = (uX_R-(uY_R*WHEELBASE/POINT_OFFSET));
vR_f = (uX_R+(uY_R*WHEELBASE/POINT_OFFSET));

if (abs(vL_f)> MAX_WHEEL_V_TICKS) || (abs(vR_f)> MAX_WHEEL_V_TICKS)
%     //determine which wheel is faster, and note the velocity
    faster_wheel_speed = max([abs(vL_f) abs(vR_f)]);
%     //use the larger wheel velocity as the divisor so that the faster
%     //wheel goes at maximum speed
    vL_f=(vL_f*MAX_WHEEL_V_TICKS)/abs(faster_wheel_speed); 
    vR_f=(vR_f*MAX_WHEEL_V_TICKS)/abs(faster_wheel_speed);
end


l_speed = fix(vL_f);
r_speed = fix(vR_f);
wheel_speed = [l_speed; r_speed];
end